import { Component, Input, ElementRef, Renderer } from '@angular/core';
import { ImageLoader } from "../../providers/image-loader";
import { ImageLoaderConfig } from "../../providers/image-loader-config";
export var ImgLoader = (function () {
    function ImgLoader(element, renderer, imageLoader, config) {
        this.element = element;
        this.renderer = renderer;
        this.imageLoader = imageLoader;
        this.config = config;
        /**
         * Fallback URL to load when the image url fails to load or does not exist.
         */
        this.fallbackUrl = this.config.fallbackUrl;
        /**
         * Whether to show a spinner while the image loads
         */
        this.spinner = this.config.spinnerEnabled;
        this._useImg = this.config.useImg;
        /**
         * Enable/Disable caching
         * @type {boolean}
         */
        this.cache = true;
        /**
         * Width of the image. This will be ignored if using useImg.
         */
        this.width = this.config.width;
        /**
         * Height of the image. This will be ignored if using useImg.
         */
        this.height = this.config.height;
        /**
         * Display type of the image. This will be ignored if using useImg.
         */
        this.display = this.config.display;
        /**
         * Background size. This will be ignored if using useImg.
         */
        this.backgroundSize = this.config.backgroundSize;
        /**
         * Background repeat. This will be ignored if using useImg.
         */
        this.backgroundRepeat = this.config.backgroundRepeat;
        /**
         * Indicates if the image is still loading
         * @type {boolean}
         */
        this.isLoading = true;
    }
    Object.defineProperty(ImgLoader.prototype, "src", {
        get: function () {
            return this._src;
        },
        /**
         * The URL of the image to load.
         */
        set: function (imageUrl) {
            this._src = this.processImageUrl(imageUrl);
            this.updateImage(this._src);
        },
        enumerable: true,
        configurable: true
    });
    ;
    Object.defineProperty(ImgLoader.prototype, "useImg", {
        /**
         * Use <img> tag
         */
        set: function (val) {
            this._useImg = val !== false;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ImgLoader.prototype, "noCache", {
        /**
         * Convenience attribute to disable caching
         * @param val
         */
        set: function (val) {
            this.cache = val !== false;
        },
        enumerable: true,
        configurable: true
    });
    ImgLoader.prototype.ngOnInit = function () {
        if (!this.src) {
            // image url was not passed
            // this can happen when [src] is set to a variable that turned out to be undefined
            // one example could be a list of users with their profile pictures
            // in this case, it would be useful to use the fallback image instead
            if (this.fallbackUrl) {
                // we're not going to cache the fallback image since it should be locally saved
                this.setImage(this.fallbackUrl);
            }
            else {
                this.isLoading = false;
            }
        }
    };
    ImgLoader.prototype.updateImage = function (imageUrl) {
        var _this = this;
        this.imageLoader.getImagePath(imageUrl)
            .then(function (imageUrl) { return _this.setImage(imageUrl); })
            .catch(function (error) { return _this.setImage(_this.fallbackUrl || imageUrl); });
    };
    /**
     * Gets the image URL to be loaded and disables caching if necessary
     * @returns {string}
     */
    ImgLoader.prototype.processImageUrl = function (imageUrl) {
        if (this.cache === false) {
            // need to disable caching
            if (imageUrl.indexOf('?') === -1) {
                imageUrl += '?';
            }
            if (['&', '?'].indexOf(imageUrl.charAt(imageUrl.length)) === -1) {
                imageUrl += '&';
            }
            // append timestamp at the end to make URL unique
            imageUrl += 'cache_buster=' + Date.now();
        }
        return imageUrl;
    };
    /**
     * Set the image to be displayed
     * @param imageUrl
     */
    ImgLoader.prototype.setImage = function (imageUrl) {
        this.isLoading = false;
        if (this._useImg) {
            // Using <img> tag
            if (!this.imgElement) {
                // create img element if we dont have one
                this.imgElement = this.renderer.createElement(this.element.nativeElement, 'img');
            }
            // set it's src
            this.renderer.setElementAttribute(this.imgElement, 'src', imageUrl);
        }
        else {
            // Not using <img> tag
            var element = this.element.nativeElement;
            if (this.display) {
                this.renderer.setElementStyle(element, 'display', this.display);
            }
            if (this.height) {
                this.renderer.setElementStyle(element, 'height', this.height);
            }
            if (this.width) {
                this.renderer.setElementStyle(element, 'width', this.width);
            }
            if (this.backgroundSize) {
                this.renderer.setElementStyle(element, 'background-size', this.backgroundSize);
            }
            if (this.backgroundRepeat) {
                this.renderer.setElementStyle(element, 'background-repeat', this.backgroundRepeat);
            }
            this.renderer.setElementStyle(element, 'background-image', 'url(\'' + imageUrl + '\')');
        }
    };
    ImgLoader.decorators = [
        { type: Component, args: [{
                    selector: 'img-loader',
                    template: '<image-loader-spinner *ngIf="spinner && isLoading"></image-loader-spinner>'
                },] },
    ];
    /** @nocollapse */
    ImgLoader.ctorParameters = [
        { type: ElementRef, },
        { type: Renderer, },
        { type: ImageLoader, },
        { type: ImageLoaderConfig, },
    ];
    ImgLoader.propDecorators = {
        'src': [{ type: Input },],
        'fallbackUrl': [{ type: Input, args: ['fallback',] },],
        'spinner': [{ type: Input },],
        'useImg': [{ type: Input },],
        'noCache': [{ type: Input },],
        'cache': [{ type: Input },],
        'width': [{ type: Input },],
        'height': [{ type: Input },],
        'display': [{ type: Input },],
        'backgroundSize': [{ type: Input },],
        'backgroundRepeat': [{ type: Input },],
    };
    return ImgLoader;
}());
//# sourceMappingURL=img-loader.js.map