import { Injectable } from '@angular/core';
import { File, Transfer } from 'ionic-native';
import { ImageLoaderConfig } from "./image-loader-config";
import * as _ from 'lodash';
export var ImageLoader = (function () {
    function ImageLoader(config) {
        var _this = this;
        this.config = config;
        /**
         * Indicates if the cache service is ready.
         * When the cache service isn't ready, images are loaded via browser instead.
         * @type {boolean}
         */
        this.isCacheReady = false;
        /**
         * Indicates if this service is initialized.
         * This service is initialized once all the setup is done.
         * @type {boolean}
         */
        this.isInit = false;
        /**
         * Number of concurrent requests allowed
         * @type {number}
         */
        this.concurrency = 5;
        /**
         * Queue items
         * @type {Array}
         */
        this.queue = [];
        this.processing = 0;
        this.cacheIndex = [];
        this.currentCacheSize = 0;
        this.indexed = false;
        if (window.location.protocol === 'http:' || window.location.protocol === 'https:') {
            // we are running on a browser, or using livereload
            // plugin will not function in this case
            this.isInit = true;
            this.throwWarning('You are running on a browser or using livereload, IonicImageLoader will not function, falling back to browser loading.');
        }
        else {
            document.addEventListener('deviceready', function () {
                _this.initCache();
            }, false);
        }
    }
    Object.defineProperty(ImageLoader.prototype, "shouldIndex", {
        get: function () {
            return (this.config.maxCacheAge > -1) || (this.config.maxCacheSize > -1);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Preload an image
     * @param imageUrl {string} Image URL
     * @returns {Promise<string>} returns a promise that resolves with the cached image URL
     */
    ImageLoader.prototype.preload = function (imageUrl) {
        return this.getImagePath(imageUrl);
    };
    /**
     * Clears the cache
     */
    ImageLoader.prototype.clearCache = function () {
        var _this = this;
        if (typeof cordova === 'undefined')
            return;
        var clear = function () {
            if (!_this.isInit) {
                // do not run this method until our service is initialized
                setTimeout(clear.bind(_this), 500);
                return;
            }
            // pause any operations
            _this.isInit = false;
            File.removeRecursively(cordova.file.cacheDirectory, _this.config.cacheDirectoryName)
                .then(function () {
                _this.initCache(true);
            })
                .catch(_this.throwError.bind(_this));
        };
        clear();
    };
    /**
     * Downloads an image via cordova-plugin-file-transfer
     * @param imageUrl {string} The remote URL of the image
     * @param localPath {string} The local path to store the image at
     * @returns {Promise<any>} Returns a promise that resolves when the download is complete, or rejects on error.
     */
    ImageLoader.prototype.downloadImage = function (imageUrl, localPath) {
        var transfer = new Transfer();
        return transfer.download(imageUrl, localPath);
    };
    /**
     * Gets the filesystem path of an image.
     * This will return the remote path if anything goes wrong or if the cache service isn't ready yet.
     * @param imageUrl {string} The remote URL of the image
     * @returns {Promise<string>} Returns a promise that will always resolve with an image URL
     */
    ImageLoader.prototype.getImagePath = function (imageUrl) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var getImage = function () {
                _this.getCachedImagePath(imageUrl)
                    .then(resolve)
                    .catch(function () {
                    // image doesn't exist in cache, lets fetch it and save it
                    _this.addItemToQueue(imageUrl, resolve, reject);
                });
            };
            var check = function () {
                if (_this.isInit) {
                    if (_this.isCacheReady) {
                        getImage();
                    }
                    else {
                        _this.throwWarning('The cache system is not running. Images will be loaded by your browser instead.');
                        resolve(imageUrl);
                    }
                }
                else {
                    setTimeout(function () { return check(); }, 250);
                }
            };
            check();
        });
    };
    /**
     * Add an item to the queue
     * @param imageUrl
     * @param resolve
     * @param reject
     */
    ImageLoader.prototype.addItemToQueue = function (imageUrl, resolve, reject) {
        this.queue.push({
            imageUrl: imageUrl,
            resolve: resolve,
            reject: reject
        });
        this.processQueue();
    };
    Object.defineProperty(ImageLoader.prototype, "canProcess", {
        /**
         * Check if we can process more items in the queue
         * @returns {boolean}
         */
        get: function () {
            return (this.queue.length > 0
                && this.processing < this.concurrency);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Processes one item from the queue
     */
    ImageLoader.prototype.processQueue = function () {
        var _this = this;
        // make sure we can process items first
        if (!this.canProcess)
            return;
        // increase the processing number
        this.processing++;
        // take the first item from queue
        var currentItem = this.queue.splice(0, 1)[0];
        // process more items concurrently if we can
        if (this.canProcess)
            this.processQueue();
        // function to call when done processing this item
        // this will reduce the processing number
        // then will execute this function again to process any remaining items
        var done = function () {
            _this.processing--;
            _this.processQueue();
        };
        var localPath = cordova.file.cacheDirectory + this.config.cacheDirectoryName + '/' + this.createFileName(currentItem.imageUrl);
        this.downloadImage(currentItem.imageUrl, localPath)
            .then(function (file) {
            if (_this.shouldIndex) {
                _this.addFileToIndex(file).then(_this.maintainCacheSize.bind(_this));
            }
            currentItem.resolve(localPath);
            done();
        })
            .catch(function (e) {
            currentItem.reject();
            _this.throwError(e);
            done();
        });
    };
    /**
     * Initialize the cache service
     * @param replace {boolean} Whether to replace the cache directory if it already exists
     */
    ImageLoader.prototype.initCache = function (replace) {
        var _this = this;
        this.concurrency = this.config.concurrency;
        if (!this.filePluginExists) {
            this.isInit = true;
            return;
        }
        this.cacheDirectoryExists
            .catch(function () {
            // doesn't exist
            return _this.createCacheDirectory(replace)
                .catch(function (e) {
                _this.throwError(e);
                _this.isInit = true;
            });
        })
            .then(function () { return _this.indexCache(); })
            .then(function () {
            _this.isCacheReady = true;
            _this.isInit = true;
        });
    };
    /**
     * Adds a file to index.
     * Also deletes any files if they are older than the set maximum cache age.
     * @param file {FileEntry} File to index
     * @returns {Promise<any>}
     */
    ImageLoader.prototype.addFileToIndex = function (file) {
        var _this = this;
        return new Promise(function (resolve, reject) { return file.getMetadata(resolve, reject); })
            .then(function (metadata) {
            if (_this.config.maxCacheAge > -1
                && (Date.now() - metadata.modificationTime.getTime()) > _this.config.maxCacheAge) {
                // file age exceeds maximum cache age
                return File.removeFile(cordova.file.cacheDirectory + _this.config.cacheDirectoryName, file.name);
            }
            else {
                // file age doesn't exceed maximum cache age, or maximum cache age isn't set
                _this.currentCacheSize += metadata.size;
                // add item to index
                _this.cacheIndex.push({
                    name: file.name,
                    modificationTime: metadata.modificationTime,
                    size: metadata.size
                });
                return Promise.resolve();
            }
        });
    };
    /**
     * Indexes the cache if necessary
     * @returns {any}
     */
    ImageLoader.prototype.indexCache = function () {
        var _this = this;
        // only index if needed, to save resources
        if (!this.shouldIndex)
            return Promise.resolve();
        this.cacheIndex = [];
        return File.listDir(cordova.file.cacheDirectory, this.config.cacheDirectoryName)
            .then(function (files) { return Promise.all(files.map(_this.addFileToIndex.bind(_this))); })
            .then(function () {
            _this.cacheIndex = _.sortBy(_this.cacheIndex, 'modificationTime');
            _this.indexed = true;
            return Promise.resolve();
        })
            .catch(function (e) {
            _this.throwError(e);
            return Promise.resolve();
        });
    };
    /**
     * This method runs every time a new file is added.
     * It checks the cache size and ensures that it doesn't exceed the maximum cache size set in the config.
     * If the limit is reached, it will delete old images to create free space.
     */
    ImageLoader.prototype.maintainCacheSize = function () {
        if (this.config.maxCacheSize > -1 && this.indexed) {
            // we exceeded max cache size
            while (this.currentCacheSize > this.config.maxCacheSize) {
                var file = this.cacheIndex.splice(0, 1)[0];
                File.removeFile(cordova.file.cacheDirectory + this.config.cacheDirectoryName, file.name);
                this.currentCacheSize -= file.size;
            }
        }
    };
    /**
     * Get the local path of a previously cached image if exists
     * @param url {string} The remote URL of the image
     * @returns {Promise<string>} Returns a promise that resolves with the local path if exists, or rejects if doesn't exist
     */
    ImageLoader.prototype.getCachedImagePath = function (url) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            // make sure cache is ready
            if (!_this.isCacheReady) {
                return reject();
            }
            // get file name
            var fileName = _this.createFileName(url);
            // get full path
            var dirPath = cordova.file.cacheDirectory + _this.config.cacheDirectoryName;
            // check if exists
            File.resolveLocalFilesystemUrl(dirPath + '/' + fileName)
                .then(function (fileEntry) {
                // file exists in cache
                // now check if iOS device & using WKWebView Engine
                if (cordova.platformId == 'ios' && window.webkit) {
                    // Read FileEntry and return as data url
                    fileEntry.file(function (file) {
                        var reader = new FileReader();
                        reader.onloadend = function () {
                            resolve(this.result);
                        };
                        reader.readAsDataURL(file);
                    }, reject);
                }
                else {
                    // return native path
                    resolve(fileEntry.nativeURL);
                }
            })
                .catch(reject); // file doesn't exist
        });
    };
    /**
     * Throws a console error if debug mode is enabled
     * @param error {string} Error message
     */
    ImageLoader.prototype.throwError = function (error) {
        if (this.config.debugMode) {
            console.error('ImageLoader Error', error);
        }
    };
    /**
     * Throws a console warning if debug mode is enabled
     * @param error {string} Error message
     */
    ImageLoader.prototype.throwWarning = function (error) {
        if (this.config.debugMode) {
            console.warn('ImageLoader Warning', error);
        }
    };
    Object.defineProperty(ImageLoader.prototype, "filePluginExists", {
        /**
         * Check if file plugin exists
         * @returns {boolean} returns a boolean that indicates whether the plugin exists
         */
        get: function () {
            if (!cordova || !cordova.file) {
                this.throwWarning('Unable to find the cordova file plugin. ImageLoader will not cache images.');
                return false;
            }
            return true;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ImageLoader.prototype, "cacheDirectoryExists", {
        /**
         * Check if the cache directory exists
         * @returns {Promise<boolean|FileError>} Returns a promise that resolves if exists, and rejects if it doesn't
         */
        get: function () {
            return File.checkDir(cordova.file.cacheDirectory, this.config.cacheDirectoryName);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Creates the cache directory
     * @param replace {boolean} override directory if exists
     * @returns {Promise<DirectoryEntry|FileError>} Returns a promise that resolves if the directory was created, and rejects on error
     */
    ImageLoader.prototype.createCacheDirectory = function (replace) {
        if (replace === void 0) { replace = false; }
        return File.createDir(cordova.file.cacheDirectory, this.config.cacheDirectoryName, replace);
    };
    /**
     * Creates a unique file name out of the URL
     * @param url {string} URL of the file
     * @returns {string} Unique file name
     */
    ImageLoader.prototype.createFileName = function (url) {
        // hash the url to get a unique file name
        return this.hashString(url).toString();
    };
    /**
     * Converts a string to a unique 32-bit int
     * @param string {string} string to hash
     * @returns {number} 32-bit int
     */
    ImageLoader.prototype.hashString = function (string) {
        var hash = 0;
        var char;
        if (string.length === 0)
            return hash;
        for (var i = 0; i < string.length; i++) {
            char = string.charCodeAt(i);
            hash = ((hash << 5) - hash) + char;
            hash = hash & hash;
        }
        return hash;
    };
    ImageLoader.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    ImageLoader.ctorParameters = [
        { type: ImageLoaderConfig, },
    ];
    return ImageLoader;
}());
//# sourceMappingURL=image-loader.js.map